package interpreter;

import environment.RuntimeEnvironment;
import error.*;
import function.builtin.EVAL;
import parser.LispParser;
import sexpression.SExpression;

public class LispInterpreter {

    protected RuntimeEnvironment environment;
    protected ErrorManager errorManager;
    private LispParser parser;

    public LispInterpreter() {
        this.environment = RuntimeEnvironment.getInstance();
        this.errorManager = this.environment.getErrorManager();
        this.parser = new LispParser(this.environment.getInput(), this.environment.getInputName());
    }

    public void interpret() {
        printGreeting();

        for (displayPrompt(); !parser.isEof(); displayPrompt())
            printValueOfNextSExpression();

        printFarewell();
    }

    protected void printGreeting() {}

    protected void displayPrompt() {}

    private void printValueOfNextSExpression() {
        try {
            printValueOfNextSExpressionWithException();
        } catch (LispException e) {
            erasePrompt();
            errorManager.handle(e);
        }
    }

    private void printValueOfNextSExpressionWithException() {
        SExpression sExpression = parser.getNextSExpression();
        String result = environment.decorateValueOutput(String.valueOf(EVAL.eval(sExpression)));

        erasePrompt();
        environment.getOutput().println(result);
    }

    protected void erasePrompt() {}

    protected void printFarewell() {
        environment.getOutput().println();
    }

}
